@@ -27,6 +27,7 @@
 #include "mlt_cache.h"
 #include "mlt_factory.h"
 #include "mlt_log.h"
+#include "mlt_producer.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -66,7 +67,6 @@ static void mlt_service_disconnect( mlt_service this );
 static void mlt_service_connect( mlt_service this, mlt_service that );
 static int service_get_frame( mlt_service this, mlt_frame_ptr frame, int index );
 static void mlt_service_property_changed( mlt_listener, mlt_properties owner, mlt_service this, void **args );
-static void purge_cache( mlt_service self );
 
 /** Initialize a service.
  *
@@ -452,6 +452,7 @@ int mlt_service_get_frame( mlt_service this, mlt_frame_ptr frame, int index )
 		mlt_properties properties = MLT_SERVICE_PROPERTIES( this );
 		mlt_position in = mlt_properties_get_position( properties, "in" );
 		mlt_position out = mlt_properties_get_position( properties, "out" );
+		mlt_position position = mlt_service_identify( this ) == producer_type ? mlt_producer_position( MLT_PRODUCER( this ) ) : -1;
 
 		result = this->get_frame( this, frame, index );
 
@@ -459,6 +460,7 @@ int mlt_service_get_frame( mlt_service this, mlt_frame_ptr frame, int index )
 		{
 			mlt_properties_inc_ref( properties );
 			properties = MLT_FRAME_PROPERTIES( *frame );
+			
 			if ( in >=0 && out > 0 )
 			{
 				mlt_properties_set_position( properties, "in", in );
@@ -466,6 +468,34 @@ int mlt_service_get_frame( mlt_service this, mlt_frame_ptr frame, int index )
 			}
 			mlt_service_apply_filters( this, *frame, 1 );
 			mlt_deque_push_back( MLT_FRAME_SERVICE_STACK( *frame ), this );
+			
+			if ( mlt_service_identify( this ) == producer_type &&
+			     mlt_properties_get_int( MLT_SERVICE_PROPERTIES( this ), "_need_previous_next" ) )
+			{
+				// Save the new position from this->get_frame
+				mlt_position new_position = mlt_producer_position( MLT_PRODUCER( this ) );
+				
+				// Get the preceding frame, unfiltered
+				mlt_frame previous_frame;
+				mlt_producer_seek( MLT_PRODUCER(this), position - 1 );
+				result = this->get_frame( this, &previous_frame, index );
+				if ( !result )
+					mlt_properties_set_data( properties, "previous frame",
+						previous_frame, 0, ( mlt_destructor ) mlt_frame_close, NULL );
+
+				// Get the following frame, unfiltered
+				mlt_frame next_frame;
+				mlt_producer_seek( MLT_PRODUCER(this), position + 1 );
+				result = this->get_frame( this, &next_frame, index );
+				if ( !result )
+				{
+					mlt_properties_set_data( properties, "next frame",
+						next_frame, 0, ( mlt_destructor ) mlt_frame_close, NULL );
+				}
+				
+				// Restore the new position
+				mlt_producer_seek( MLT_PRODUCER(this), new_position );
+			}
 		}
 	}
 
@@ -525,7 +555,9 @@ int mlt_service_attach( mlt_service this, mlt_filter filter )
 				mlt_properties props = MLT_FILTER_PROPERTIES( filter );
 				mlt_properties_inc_ref( MLT_FILTER_PROPERTIES( filter ) );
 				base->filters[ base->filter_count ++ ] = filter;
+				mlt_properties_set_data( props, "service", this, 0, NULL, NULL );
 				mlt_events_fire( properties, "service-changed", NULL );
+				mlt_events_fire( props, "service-changed", NULL );
 				mlt_events_listen( props, this, "service-changed", ( mlt_listener )mlt_service_filter_changed );
 				mlt_events_listen( props, this, "property-changed", ( mlt_listener )mlt_service_filter_changed );
 			}
@@ -635,7 +667,6 @@ void mlt_service_close( mlt_service this )
 			free( base->in );
 			pthread_mutex_destroy( &base->mutex );
 			free( base );
-			purge_cache( this );
 			mlt_properties_close( &this->parent );
 		}
 	}
@@ -651,7 +682,7 @@ void mlt_service_close( mlt_service this )
  * \param self a service
  */
 
-static void purge_cache( mlt_service self )
+void mlt_service_cache_purge( mlt_service self )
 {
 	mlt_properties caches = mlt_properties_get_data( mlt_global_properties(), "caches", NULL );
 
@@ -736,3 +767,18 @@ mlt_cache_item mlt_service_cache_get( mlt_service self, const char *name )
 
 	return result;
 }
+
+/** Set the number of items to cache for the named cache.
+ *
+ * \public \memberof mlt_service_s
+ * \param self a service
+ * \param name a name for the object that is unique to the service class, but not to the instance
+ * \param size the number of items to cache
+ */
+
+void mlt_service_cache_set_size( mlt_service self, const char *name, int size )
+{
+	mlt_cache cache = get_cache( self, name );
+	if ( cache )
+		mlt_cache_set_size( cache, size );
+}
