@@ -156,7 +156,8 @@ struct consumer_SDIstream_s {
 	int height;
 
 	struct audio_format audio_format;
-	/** device file:
+	/**
+	 * device file:
 	 *		/dev/sditx0
 	 *		/dev/sdivideotx0
 	 *		/dev/sdiaudiotx0
@@ -164,14 +165,15 @@ struct consumer_SDIstream_s {
 	char *device_file_video; // Path for SDI output
 	char *device_file_audio; // Path for exlusive SDI audio output
 
-	/* write own HANC (ancillary data) is available for:
+	/**
+	 *  write own HANC (ancillary data) is available for:
 	 * SDI board ASSY 193 HD        'blanking=false'
 	 * SDI board ASSY 180 SD quad   'blanking=true'
 	 * SDI board ASSY 145 SD single 'blanking=true'
 	 *
 	 * 0=false, 1=true
 	 *
-	 * */
+	 **/
 	uint8_t blanking;
 
 	// our audio channel pair for this frame
@@ -191,7 +193,7 @@ static void consumer_close(mlt_consumer parent);
 static void *consumer_thread(void *);
 
 static void consumer_write_JPEG(char * path, uint8_t **vBuffer, mlt_profile myProfile);
-int convertYCBCRtoRGB(int y1, int cb, int cr, int y2, int * target_rgb);
+int convertYCBCRtoRGB(int y1, int cb, int cr, int y2, uint8_t * target_rgb);
 
 /*****************************************************************************************************
  ****************************************** SDI Master Consumer **************************************
@@ -229,7 +231,7 @@ mlt_consumer consumer_SDIstream_init(mlt_profile profile, mlt_service_type type,
 		parent->stop = consumer_stop;
 		parent->is_stopped = consumer_is_stopped;
 
-		// Set explizit to zero or other value
+		// Set explicit to zero or other value
 		int i, j;
 		for (i = 0; i < MAX_AUDIO_STREAMS; i++) {
 			for (j = 0; j < MAX_AUDIO_SAMPLES; j++) {
@@ -368,10 +370,6 @@ static void *consumer_thread(void *arg) {
 	// Define a frame pointer
 	mlt_frame frame;
 
-	if (!sdi_init(this->device_file_video, this->device_file_audio, this->blanking, mlt_service_profile((mlt_service) consumer))) {
-		exit(0);
-	}
-
 	// set Datablock number for SDI encoding
 	int my_dbn = 1;
 
@@ -379,14 +377,20 @@ static void *consumer_thread(void *arg) {
 	unsigned int count = 0;
 
 	// Tell the framework how we want our audio and video
-	int frequency = 48000;
+	int frequency = this->audio_format.sample_rate;
 	int channels = 0;
 	int samples = mlt_sample_calculator(fps, frequency, count++);
 
-	this->audio_format.aformat = mlt_audio_s16;
-	this->audio_format.channels = 0;
+	// set number of audio channels, linsys vidport model 193 is limited to 8 channels (4AES frames)
+	this->audio_format.channels = 8; /* 0,2,4,6,8 */
+	this->audio_format.aformat = mlt_audio_s16; /* 16, 24, 32 */
+	this->audio_format.sample_rate = 48000;
 	this->pix_fmt = mlt_image_yuv422;
 
+	if (!sdi_init(this->device_file_video, this->device_file_audio, this->blanking, mlt_service_profile((mlt_service) consumer), &this->audio_format)) {
+		exit(0);
+	}
+
 	uint8_t *video_buffer;
 	int16_t *audio_buffer_tmp; // the upstream audio buffer
 
@@ -417,10 +421,10 @@ static void *consumer_thread(void *arg) {
 				this->audio_format.sample_rate = frequency;
 				this->audio_format.samples = samples;
 
-				// TODO, current hard coded to all 8 channels, also we write 8 channels to the sdi board
-				// the Linys SDI board must be right configured with the same number of channels!!!
-				this->audio_format.channels = 8;
-				// this->audio_format.channels = channels; // take given number of channels
+				/* TODO: Audio is currently hard coded to 8 channels because write 8 channels to the sdi board.
+				 The Linys SDI board has to be configured with the same number of channels!
+				 this->audio_format.channels = channels; // take given number of channels
+				 */
 
 				/* Tell the sdi_generator.c to playout our frame
 				 * 8 AES (8 x stereo channels are possible, max. 16 channels) Linsys SD board model:	107, 159, 145, 180
@@ -579,69 +583,42 @@ static void consumer_write_JPEG(char * filename, uint8_t **vBuffer, mlt_profile
  * @param target pointer
  * @return 0 upon success
  **/
-int convertYCBCRtoRGB(int y1, int cb, int cr, int y2, int * target_rgb) {
+int convertYCBCRtoRGB(int y1, int cb, int cr, int y2, uint8_t * target_rgb) {
 
 #ifdef WITH_JPEG
 
-	uint32_t tmp = 0;
+	if(y1 > 235)
+		y1 = 235;
+	if(y1 < 16)
+		y1 = 16;
+
+	if(y2 > 235)
+		y2 = 235;
+	if(y2 < 16)
+		y2 = 16;
+
+	if(cr > 240)
+		cr = 240;
+	if(cr < 16)
+		cr = 16;
+
+	if(cb > 240)
+		cb = 240;
+	if(cb < 16)
+		cb = 16;
+
 	uint8_t r1, g1, b1, r2, g2, b2;
 
 	//pointer to current output buffer position
 	uint8_t * target_pointer = target_rgb;
 
-	// colorconversion for first pixel and output to rgb buffer
-	/* B1 */
-	tmp = 1.164 * (y1 - 16) + 2.017 * (cb - 128);
-	if (tmp > 255)
-		tmp = 255;
-	if (tmp < 0)
-		tmp = 0;
-	b1 = tmp;
+	r1 = y1 + 1.402 * (cr - 128);
+	g1 = y1 - 0.34414 * (cb - 128) - 0.71414 * (cr - 128);
+	b1 = y1 + 1.772 * (cb - 128);
 
-	/* G1 */
-	tmp = 1.164 * (y1 - 16) - 0.813 * (cr - 128) - 0.392 * (cb - 128);
-	if (tmp > 255)
-		tmp = 255;
-	if (tmp < 0)
-		tmp = 0;
-	g1 = tmp;
-
-	/* R1 */
-	tmp = 1.164 * (y1 - 16) + 1.596 * (cr - 128);
-	if (tmp > 255)
-		tmp = 255;
-	if (tmp < 0)
-		tmp = 0;
-	r1 = tmp;
-
-	// colorconversion for second pixel and output to rgb buffer
-	/* B2 */
-	tmp = 1.164 * (y2 - 16) + 2.017 * (cb - 128);
-	if (tmp > 255)
-		tmp = 255;
-	if (tmp < 0)
-		tmp = 0;
-	b2 = tmp;
-
-	/* G2 */
-	tmp = 1.164 * (y2 - 16) - 0.813 * (cr - 128) - 0.392 * (cb - 128);
-	if (tmp > 255)
-		tmp = 255;
-	if (tmp < 0)
-		tmp = 0;
-	g2 = tmp;
-
-	/* R2 */
-	tmp = 1.164 * (y2 - 16) + 1.596 * (cr - 128);
-	if (tmp > 255)
-		tmp = 255;
-	if (tmp < 0)
-		tmp = 0;
-	r2 = tmp;
-
-	//	r=y+1.5958*v;
-	//  g=y-0.39173*u-0.81290*v;
-	//  b=y+2.017*u;
+	r2 = y2 + 1.402 * (cr - 128);
+	g2 = y2 - 0.34414 * (cb - 128) - 0.71414 *(cr - 128);
+	b2 = y2 + 1.772 * (cb - 128);
 
 	*target_pointer++ = r1;
 	*target_pointer++ = g1;
