@@ -37,10 +37,12 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 	mlt_properties properties = MLT_FILTER_PROPERTIES( filter );
 
 	// Get the image
-	int error = 0; //mlt_frame_get_image( this, image, format, width, height, 0 );
+	int error = 0;
+	*format = mlt_image_yuv422;
+	//mlt_frame_get_image( this, image, format, width, height, 0 );
 
 	// Only process if we have no error and a valid colour space
-	if ( error == 0 && *format == mlt_image_yuv422 )
+	if ( error == 0 )
 	{
 		mlt_producer producer = mlt_properties_get_data( properties, "producer", NULL );
 		mlt_transition transition = mlt_properties_get_data( properties, "transition", NULL );
