@@ -115,6 +115,7 @@ mlt_consumer consumer_sdl_init( mlt_profile profile, mlt_service_type type, cons
 		
 		// Default scaler (for now we'll use nearest)
 		mlt_properties_set( this->properties, "rescale", "nearest" );
+		mlt_properties_set( this->properties, "deinterlace_method", "onefield" );
 
 		// Default buffer for low latency
 		mlt_properties_set_int( this->properties, "buffer", 1 );
@@ -188,7 +189,7 @@ int consumer_start( mlt_consumer parent )
 		{
 			if ( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE ) < 0 )
 			{
-				fprintf( stderr, "Failed to initialize SDL: %s\n", SDL_GetError() );
+				mlt_log_error( MLT_CONSUMER_SERVICE(parent), "Failed to initialize SDL: %s\n", SDL_GetError() );
 				return -1;
 			}
 
@@ -346,6 +347,7 @@ static int consumer_play_audio( consumer_sdl this, mlt_frame frame, int init_aud
 
 	// Set the preferred params of the test card signal
 	int channels = mlt_properties_get_int( properties, "channels" );
+	int dest_channels = channels;
 	int frequency = mlt_properties_get_int( properties, "frequency" );
 	static int counter = 0;
 
@@ -356,6 +358,7 @@ static int consumer_play_audio( consumer_sdl this, mlt_frame frame, int init_aud
 
 	mlt_frame_get_audio( frame, (void**) &pcm, &afmt, &frequency, &channels, &samples );
 	*duration = ( ( samples * 1000 ) / frequency );
+	pcm += mlt_properties_get_int( properties, "audio_offset" );
 
 	if ( mlt_properties_get_int( properties, "audio_off" ) )
 	{
@@ -376,7 +379,7 @@ static int consumer_play_audio( consumer_sdl this, mlt_frame frame, int init_aud
 		this->playing = 0;
 		request.freq = frequency;
 		request.format = AUDIO_S16SYS;
-		request.channels = channels;
+		request.channels = dest_channels;
 		request.samples = audio_buffer;
 		request.callback = sdl_fill_audio;
 		request.userdata = (void *)this;
@@ -395,16 +398,36 @@ static int consumer_play_audio( consumer_sdl this, mlt_frame frame, int init_aud
 	if ( init_audio == 0 )
 	{
 		mlt_properties properties = MLT_FRAME_PROPERTIES( frame );
-		bytes = ( samples * channels * 2 );
+		
+		bytes = samples * dest_channels * sizeof(*pcm);
 		pthread_mutex_lock( &this->audio_mutex );
 		while ( this->running && bytes > ( sizeof( this->audio_buffer) - this->audio_avail ) )
 			pthread_cond_wait( &this->audio_cond, &this->audio_mutex );
 		if ( this->running )
 		{
 			if ( mlt_properties_get_double( properties, "_speed" ) == 1 )
-				memcpy( &this->audio_buffer[ this->audio_avail ], pcm, bytes );
+			{
+				if ( channels == dest_channels )
+				{
+					memcpy( &this->audio_buffer[ this->audio_avail ], pcm, bytes );
+				}
+				else
+				{
+					int16_t *dest = (int16_t*) &this->audio_buffer[ this->audio_avail ];
+					int i = samples + 1;
+					
+					while ( --i )
+					{
+						memcpy( dest, pcm, dest_channels * sizeof(*pcm) );
+						pcm += channels;
+						dest += dest_channels;
+					}
+				}
+			}
 			else
+			{
 				memset( &this->audio_buffer[ this->audio_avail ], 0, bytes );
+			}
 			this->audio_avail += bytes;
 		}
 		pthread_cond_broadcast( &this->audio_cond );
@@ -677,6 +700,10 @@ static void *video_thread( void *arg )
 				start = ( ( int64_t )now.tv_sec * 1000000 + now.tv_usec ) - scheduled + 20000;
 			}
 		}
+		else
+		{
+			mlt_log_debug( MLT_CONSUMER_SERVICE(&this->parent), "dropped video frame\n" );
+		}
 
 		// This frame can now be closed
 		mlt_frame_close( next );
